﻿using System;
using System.Globalization;
using Mediachase.Commerce.Orders;

namespace EPiServer.Business.Commerce.Payment.Klarna
{
    /// <summary>
    /// Simple address struct for Klarna
    /// </summary>
    [Serializable]
    public class Address
    {
        /// <summary>
        /// Gets or sets the Last Name
        /// </summary>
        /// <value>The Last name.</value>
        public string LastName { get; set; }

        /// <summary>
        /// Gets or sets the street.
        /// </summary>
        /// <value>The street.</value>
        public string Street { get; set; }

        /// <summary>
        /// Gets or sets the zip.
        /// </summary>
        /// <value>The zip.</value>
        public string Zip { get; set; }

        /// <summary>
        /// Gets or sets the city.
        /// </summary>
        /// <value>The city.</value>
        public string City { get; set; }

        /// <summary>
        /// Gets or sets the country.
        /// </summary>
        /// <value>The country.</value>
        public string Country { get; set; }

        /// <summary>
        /// Initializes a new instance of the <see cref="Address"/> class.
        /// </summary>
        public Address()
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="Address"/> struct.
        /// </summary>
        /// <param name="lname">The lname.</param>
        /// <param name="street">The street.</param>
        /// <param name="zip">The zip.</param>
        /// <param name="city">The city.</param>
        /// <param name="country">The country.</param>
        public Address(string lname, string street, string zip, string city, string country)
        {
            LastName = lname;
            Street = street;
            Zip = zip;
            City = city;
            Country = country;
        }

        /// <summary>
        /// Gets the string represent the address.
        /// </summary>
        /// <returns>
        /// A <see cref="System.String"/> that represents this instance.
        /// </returns>
        public override string ToString()
        {
            if (string.IsNullOrEmpty(LastName))
            {
                return string.Format(" {0}, {1} {2}", Street, Zip, City);
            }
            return string.Format(" {0}, {1}, {2} {3}", LastName, Street, Zip, City);
        }

        /// <summary>
        /// Compares the address in cart and address retured from Klanar system.
        /// </summary>
        /// <param name="mcAddress">The Order address (in cart).</param>
        /// <returns>True if the order address is same as klarna address </returns>
        internal bool CompareWith(OrderAddress mcAddress)
        {
            var regionInfo = new RegionInfo(Country);
            if (LastName.Trim().Equals(mcAddress.LastName.Trim(), StringComparison.InvariantCultureIgnoreCase) &&
                City.Trim().Equals(mcAddress.City.Trim(), StringComparison.InvariantCultureIgnoreCase) &&
                Zip.Trim().Equals(mcAddress.PostalCode.Trim(), StringComparison.InvariantCultureIgnoreCase) &&
                regionInfo.ThreeLetterISORegionName.Trim().Equals(mcAddress.CountryCode.Trim(), StringComparison.InvariantCultureIgnoreCase) &&
                Street.Trim().Equals(mcAddress.Line1.Trim(), StringComparison.InvariantCultureIgnoreCase))
            {
                return true;
            }
            return false;
        }
    }
}
