﻿using System;
using Mediachase.Web.Console.Interfaces;
using Mediachase.Commerce.Orders.Dto;
using System.Data;

namespace EPiServer.Business.Commerce.Payment.Klarna.KlarnaInvoice
{
    /// <summary>
    /// Configure payment for Klarna Invoice module
    /// </summary>
    public partial class ConfigurePayment : System.Web.UI.UserControl, IGatewayControl
    {
        // Fields
        private PaymentMethodDto _paymentMethodDto;

        /// <summary>
        /// Initializes a new instance of the <see cref="ConfigurePayment"/> class.
        /// </summary>
        public ConfigurePayment()
        {
            ValidationGroup = string.Empty;
            _paymentMethodDto = null;
        }

        /// <summary>
        /// Handles the Load event of the Page control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected void Page_Load(object sender, EventArgs e)
        {
            if ((_paymentMethodDto != null) && (_paymentMethodDto.PaymentMethodParameter != null))
            {
                // TestMode parameter
                var testFlagParameter = GetParameterByName(KlarnaHelper.TestFlag);
                if (testFlagParameter != null)
                {
                    bool testFlag;
                    bool.TryParse(testFlagParameter.Value, out testFlag);
                    TestFlagYes.Checked = testFlag;
                    TestFlagNo.Checked = !testFlag;
                }

                // MerchantId parameter
                var merchantIdparameter = GetParameterByName(KlarnaHelper.MerchantId);
                if (merchantIdparameter != null)
                {
                    MerchantIdText.Text = merchantIdparameter.Value;
                }

                // SharedSecret parameter
                var sharedSecretparameter = GetParameterByName(KlarnaHelper.SharedSecret);
                if (sharedSecretparameter != null)
                {
                    SharedSecretText.Text = sharedSecretparameter.Value;
                }

                // Enable different address parameter
                var enableCustomerEnterAddressParameter = GetParameterByName(KlarnaHelper.EnableCustomerEnterAddress);
                if (enableCustomerEnterAddressParameter != null)
                {
                    bool enabled;
                    bool.TryParse(enableCustomerEnterAddressParameter.Value, out enabled);
                    EnableCustomerEnterAddressCheckBox.Checked = enabled;
                }
            }
            else
            {
                Visible = false;
            }
        }

        #region IGatewayControl Members
        /// <summary>
        /// Loads the object.
        /// </summary>
        /// <param name="dto">The dto.</param>
        public void LoadObject(object dto)
        {
            _paymentMethodDto = dto as PaymentMethodDto;
        }

        /// <summary>
        /// Saves the changes.
        /// </summary>
        /// <param name="dto">The dto.</param>
        public void SaveChanges(object dto)
        {
            if (Visible)
            {
                _paymentMethodDto = dto as PaymentMethodDto;
                if ((_paymentMethodDto != null) && (_paymentMethodDto.PaymentMethodParameter != null))
                {
                    // Get the PaymentMethodId
                    var methodId = Guid.Empty;
                    if (_paymentMethodDto.PaymentMethod.Count > 0)
                    {
                        methodId = _paymentMethodDto.PaymentMethod[0].PaymentMethodId;
                    }

                    // TestFlag parameter
                    var testFlagParameter = GetParameterByName(KlarnaHelper.TestFlag);
                    if (testFlagParameter != null)
                    {
                        testFlagParameter.Value = TestFlagYes.Checked.ToString();
                    }
                    else
                    {
                        CreateParameter(_paymentMethodDto, KlarnaHelper.TestFlag, TestFlagYes.Checked.ToString(), methodId);
                    }

                    // MerchantId parameter
                    var merchantIdparameter = GetParameterByName(KlarnaHelper.MerchantId);
                    if (merchantIdparameter != null)
                    {
                        merchantIdparameter.Value = MerchantIdText.Text;
                    }
                    else
                    {
                        CreateParameter(_paymentMethodDto, KlarnaHelper.MerchantId, MerchantIdText.Text, methodId);
                    }

                    // SharedSecret parameter
                    var sharedSecretparameter = GetParameterByName(KlarnaHelper.SharedSecret);
                    if (sharedSecretparameter != null)
                    {
                        sharedSecretparameter.Value = SharedSecretText.Text;
                    }
                    else
                    {
                        CreateParameter(_paymentMethodDto, KlarnaHelper.SharedSecret, SharedSecretText.Text, methodId);
                    }

                    // SharedSecret parameter
                    var enableCustomerEnterAddressParameter = GetParameterByName(KlarnaHelper.EnableCustomerEnterAddress);
                    if (enableCustomerEnterAddressParameter != null)
                    {
                        enableCustomerEnterAddressParameter.Value = EnableCustomerEnterAddressCheckBox.Checked.ToString();
                    }
                    else
                    {
                        CreateParameter(_paymentMethodDto, KlarnaHelper.EnableCustomerEnterAddress, EnableCustomerEnterAddressCheckBox.Checked.ToString(), methodId);
                    }
                }
            }
        }

        /// <summary>
        /// Gets the name of the parameter by.
        /// </summary>
        /// <param name="parameterName">Name of the parameter.</param>
        /// <returns></returns>
        private PaymentMethodDto.PaymentMethodParameterRow GetParameterByName(string parameterName)
        {
            var rowArray = (PaymentMethodDto.PaymentMethodParameterRow[])_paymentMethodDto.PaymentMethodParameter.Select(string.Format("Parameter = '{0}'", parameterName));
            if ((rowArray != null) && (rowArray.Length > 0))
            {
                return rowArray[0];
            }

            return null;
        }

        /// <summary>
        /// Creates the parameter.
        /// </summary>
        /// <param name="dto">The dto.</param>
        /// <param name="parameterName">Name of the parameter.</param>
        /// <param name="value">The value.</param>
        /// <param name="paymentMethodId">The payment method id.</param>
        private void CreateParameter(PaymentMethodDto dto, string parameterName, string value, Guid paymentMethodId)
        {
            var row = dto.PaymentMethodParameter.NewPaymentMethodParameterRow();
            row.PaymentMethodId = paymentMethodId;
            row.Parameter = parameterName;
            row.Value = value;
            if (row.RowState == DataRowState.Detached)
            {
                dto.PaymentMethodParameter.Rows.Add(row);
            }
        }

        /// <summary>
        /// Gets or sets the validation group.
        /// </summary>
        /// <value>The validation group.</value>
        public string ValidationGroup { get; set; }

        #endregion
    }
}