﻿using Klarna.Api;
using System;

namespace EPiServer.Business.Commerce.Payment.Klarna.Services
{
    /// <summary>
    /// This class intents to hold information of PClass object which is return by Klarna service.
    /// We will store this class to our DB instead of PClass due to PClass doesn't has a none argurment constructor
    /// The none argurment constructor is a requirement when get back an object from DB using GetStore method in DynamicDataStoreFactory class
    /// </summary>
    public class EPiPClass
    {
        public int EID { get; set; }
        public int PClassID { get; set; }
        public string Description { get; set; }
        public int Months { get; set; }
        public double StartFee { get; set; }
        public double InvoiceFee { get; set; }
        public double InterestRate { get; set; }
        public double MinAmountForPClass { get; set; }
        public Country.Code Country { get; set; }
        public PClass.TypeCode Type { get; set; }
        public DateTime ExpireDate { get; set; }

        /// <summary>
        /// Initializes a new instance of the <see cref="EPiPClass"/> class.
        /// </summary>
        public EPiPClass()
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="EPiPClass"/> class.
        /// </summary>
        /// <param name="pc">The PClass.</param>
        public EPiPClass(PClass pc)
        {
            EID = pc.Eid;
            PClassID = pc.Id;
            Description = pc.Description;
            Months = pc.Months;
            StartFee = pc.StartFee;
            InvoiceFee = pc.InvoiceFee;
            InterestRate = pc.InterestRate;
            MinAmountForPClass = pc.MinimumPurchaseAmount;
            Country = pc.Country;
            Type = pc.Type;
            ExpireDate = pc.ExpiryDate;
        }

        /// <summary>
        /// Convert an EPiPClass to Klarna PClass
        /// </summary>
        /// <returns>A Klarna PClass</returns>
        public PClass ToPClass()
        {
            object[] param = { 
                this.PClassID, 
                this.Description, 
                this.Months, 
                this.StartFee, 
                this.InvoiceFee, 
                this.InterestRate, 
                this.MinAmountForPClass, 
                this.Country, 
                this.Type, 
                this.ExpireDate 
            };
            return new PClass(this.EID, param);
        }
    }
}
