namespace EPiServer.Business.Commerce.Payment.PayPal
{
    using EPiServer.Business.Commerce.Payment.PayPal.Orders;
    using EPiServer.Framework.Localization;
    using EPiServer.ServiceLocation;
    using global::PayPal.PayPalAPIInterfaceService.Model;
    using Mediachase.Commerce.Core;
    using Mediachase.Commerce.Orders;
    using Mediachase.Commerce.Orders.Dto;
    using Mediachase.Commerce.Orders.Managers;
    using Mediachase.Commerce.Website;
    using Mediachase.Commerce.Website.BaseControls;
    using System.Net;

    /// <summary>
    ///	Implements User interface for generic gateway
    /// </summary>
    public partial class PaymentMethod : BaseStoreUserControl, IPaymentOption
    {
        /// <summary>
        /// Handles the Load event of the Page control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void Page_Load(object sender, System.EventArgs e)
        {
            ServicePointManager.SecurityProtocol = SecurityProtocolType.Tls12;

            PaymentMethodDto payPal = Utilities.GetPayPalPaymentMethod();
            string pal = payPal.GetParameterValueByName(PayPalPaymentGateway.PALParameter);
            if (string.IsNullOrEmpty(pal))
            {
                pal = ViewState["PAL"] as string;
            }
            if (string.IsNullOrEmpty(pal))
            {
                //Obtain the PAL code using API
                var caller = Utilities.GetPayPalAPICallerServices();

                if (caller != null)
                {   
                    GetPalDetailsRequestType palRequest = new GetPalDetailsRequestType();
                    GetPalDetailsResponseType palResponse = new GetPalDetailsResponseType();
                    palResponse = (GetPalDetailsResponseType)caller.GetPalDetails(new GetPalDetailsReq() { GetPalDetailsRequest = palRequest });
                    if (palResponse.Ack == AckCodeType.SUCCESSWITHWARNING || palResponse.Ack == AckCodeType.SUCCESS)
                    {
                        pal = palResponse.Pal;
                        ViewState["PAL"] = pal;
                    }
                }
            }

            ImageMark.Attributes.Add("onclick", "javascript:var top= (screen.availHeight - 500)/2; var left=(screen.availWidth - 400)/2;window.open('https://www.paypal.com/se/cgi-bin/webscr?cmd=xpt/Marketing/popup/OLCWhatIsPayPal-outside','PayPalHelp', 'height=500, width=400, top=' +top +',left=' + left)");
            ImageMark.Style.Add(System.Web.UI.HtmlTextWriterStyle.Cursor, "pointer");

            if (payPal.GetParameterValueByName(PayPalPaymentGateway.SandBoxParameter) != "1")
            {
                // LIVE PayPal payment

                if (string.IsNullOrEmpty(pal))
                {
                    //Default image
                    ImageMark.ImageUrl = "https://fpdbs.paypal.com/dynamicimageweb?cmd=_dynamic-image&buttontype=ecmark";
                }
                else
                {
                    ImageMark.ImageUrl = string.Format("https://fpdbs.paypal.com/dynamicimageweb?cmd=_dynamic-image&buttontype=ecmark&pal={0}&locale={1}", pal, SiteContext.Current.LanguageName);
                }
            }
            else
            {
                // SANBOX PayPal payment

                if (string.IsNullOrEmpty(pal))
                {
                    //Default image
                    ImageMark.ImageUrl = "https://fpdbs.sandbox.paypal.com/dynamicimageweb?cmd=_dynamic-image&buttontype=ecmark";
                }
                else
                {
                    ImageMark.ImageUrl = string.Format("https://fpdbs.sandbox.paypal.com/dynamicimageweb?cmd=_dynamic-image&buttontype=ecmark&pal={0}&locale={1}", pal, SiteContext.Current.LanguageName);
                }
            }

            Message.Text = LocalizationService.GetString("/Commerce/PayPalPayment/PayPalText");

            if (string.Equals(Request["accept"], "false") && !string.IsNullOrEmpty(Request.QueryString["hash"]))
            {
                ErrorManager.GenerateError(CancelledMessage);
                return;
            }
        }

        /// <summary>
        /// Gets the message when payment is cancelled or declined.
        /// </summary>
        /// <value>The message.</value>
        protected string CancelledMessage
        {
            get
            {
                return LocalizationService.GetString("/Commerce/PayPalPayment/PaymentCancelMessage");
            }
        }

        #region IPaymentOption Members

        /// <summary>
        /// Validates input data for the control. In case of Credit card pre authentication will be the best way to
        /// validate. The error message if any should be displayed within a control itself.
        /// </summary>
        /// <returns>Returns false if validation is failed.</returns>
        public bool ValidateData()
        {
            return true;
        }

        /// <summary>
        /// This method is called before the order is completed. This method should check all the parameters
        /// and validate the credit card or other parameters accepted.
        /// </summary>
        /// <param name="form"></param>
        /// <returns>bool</returns>
        public Payment PreProcess(OrderForm form)
        {          
            PaymentMethodDto payPal = PaymentManager.GetPaymentMethodBySystemName("PayPal", SiteContext.Current.LanguageName);
            var payment = new PayPalPayment();
            string paymentAction = payPal.GetParameterValueByName(PayPalPaymentGateway.PaymentActionParameter);
            if (paymentAction == "Authorization")
            {
                payment.TransactionType = TransactionType.Authorization.ToString();
            }
            else
            {
                payment.TransactionType = TransactionType.Sale.ToString();
            }
            return payment;
        }
        
        /// <summary>
        /// This method is called after the order is placed. This method should be used by the gateways that want to
        /// redirect customer to their site.
        /// </summary>
        /// <param name="form">The form.</param>
        /// <returns></returns>
        public bool PostProcess(OrderForm form)
        {
            return true;
        }

        #endregion

        private static LocalizationService LocalizationService
        {
            get { return ServiceLocator.Current.GetInstance<LocalizationService>(); }
        }
    }
}
